﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.XPath;
using System.IO;
using System.Web.Mvc;
using System.Web;
using System.Xml.Xsl;
using System.Web.UI;

namespace SearchBloxSearch
{
    ///
    /// Provides support for rendering a XML.
    ///
    public static class RenderXmlExtensions
    {
        #region Fields
        private static XslCompiledTransform _transparentTransform;
        #endregion

        #region Constructor
        static RenderXmlExtensions()
        {
         //   XmlTextReader copyTransformReader = new XmlTextReader(new StringReader("   "));
            _transparentTransform = new XslCompiledTransform();
         //   _transparentTransform.Load(copyTransformReader);
        }
        #endregion

        #region Methods
        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///The XML document
        ///The XSLT transformation
        public static void RenderXml(this HtmlHelper htmlHelper, XmlDocument xmlDocument, XslCompiledTransform transform)
        {
            RenderXmlInternal(htmlHelper, xmlDocument, null, String.Empty, transform, String.Empty, null);
        }

        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///The XML document
        ///The XSLT transformation
        ///The XSLT transformation arguments list
        public static void RenderXml(this HtmlHelper htmlHelper, XmlDocument xmlDocument, XslCompiledTransform transform, XsltArgumentList transformArgumentList)
        {
            RenderXmlInternal(htmlHelper, xmlDocument, null, String.Empty, transform, String.Empty, transformArgumentList);
        }

        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///The XML document
        ///The XSLT transformation source
        public static void RenderXml(this HtmlHelper htmlHelper, XmlDocument xmlDocument, string transformSource)
        {
            RenderXmlInternal(htmlHelper, xmlDocument, null, String.Empty, null, transformSource, null);
        }

        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///The XML document
        ///The XSLT transformation source
        ///The XSLT transformation arguments list
        public static void RenderXml(this HtmlHelper htmlHelper, XmlDocument xmlDocument, string transformSource, XsltArgumentList transformArgumentList)
        {
            RenderXmlInternal(htmlHelper, xmlDocument, null, String.Empty, null, transformSource, transformArgumentList);
        }

        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///The XML XPath navigator
        ///The XSLT transformation
        public static void RenderXml(this HtmlHelper htmlHelper, XPathNavigator xpathNavigator, XslCompiledTransform transform)
        {
            RenderXmlInternal(htmlHelper, null, xpathNavigator, String.Empty, transform, String.Empty, null);
        }

        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///The XML XPath navigator
        ///The XSLT transformation
        ///The XSLT transformation arguments list
        public static void RenderXml(this HtmlHelper htmlHelper, XPathNavigator xpathNavigator, XslCompiledTransform transform, XsltArgumentList transformArgumentList)
        {
            RenderXmlInternal(htmlHelper, null, xpathNavigator, String.Empty, transform, String.Empty, transformArgumentList);
        }

        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///The XML XPath navigator
        ///The XSLT transformation source
        public static void RenderXml(this HtmlHelper htmlHelper, XPathNavigator xpathNavigator, string transformSource)
        {
            RenderXmlInternal(htmlHelper, null, xpathNavigator, String.Empty, null, transformSource, null);
        }

        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///The XML XPath navigator
        ///The XSLT transformation source
        ///The XSLT transformation arguments list
        public static void RenderXml(this HtmlHelper htmlHelper, XPathNavigator xpathNavigator, string transformSource, XsltArgumentList transformArgumentList)
        {
            RenderXmlInternal(htmlHelper, null, xpathNavigator, String.Empty, null, transformSource, transformArgumentList);
        }

        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///The XML path
        ///The XSLT transformation
        public static void RenderXml(this HtmlHelper htmlHelper, string documentSource, XslCompiledTransform transform)
        {
            RenderXmlInternal(htmlHelper, null, null, documentSource, transform, String.Empty, null);
        }

        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///he XML path
        ///The XSLT transformation
        ///The XSLT transformation arguments list
        public static void RenderXml(this HtmlHelper htmlHelper, string documentSource, XslCompiledTransform transform, XsltArgumentList transformArgumentList)
        {
            RenderXmlInternal(htmlHelper, null, null, documentSource, transform, String.Empty, transformArgumentList);
        }

        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///The XML path
        ///The XSLT transformation source
        public static void RenderXml(this HtmlHelper htmlHelper, string documentSource, string transformSource)
        {
            RenderXmlInternal(htmlHelper, null, null, documentSource, null, transformSource, null);
        }

        ///
        /// Renders the specified XML by using the specified HMTL helper.
        ///
        ///The HTML helper
        ///The XML path
        ///The XSLT transformation source
        ///The XSLT transformation arguments list
        public static void RenderXml(this HtmlHelper htmlHelper, string documentSource, string transformSource, XsltArgumentList transformArgumentList)
        {
            RenderXmlInternal(htmlHelper, null, null, documentSource, null, transformSource, transformArgumentList);
        }

        private static void RenderXmlInternal(HtmlHelper htmlHelper, XmlDocument xmlDocument, XPathNavigator xpathNavigator, string documentSource, XslCompiledTransform transform, string transformSource, XsltArgumentList transformArgumentList)
        {
            XPathDocument xpathDocument = null;
            HtmlTextWriter writer = new HtmlTextWriter(htmlHelper.ViewContext.Writer);
            //Checking if we have been given XmlDocument or XPathNavigator directly, or do we have path for document
            if ((xmlDocument == null) && (xpathNavigator == null) && (!String.IsNullOrEmpty(documentSource) && (documentSource.Trim().Length != 0)))
            {
                //Checking if path is absolute or relative
                if (!Path.IsPathRooted(documentSource))
                    //Mapping the relative path
                    documentSource = htmlHelper.ViewContext.HttpContext.Server.MapPath(documentSource);

                //Loading XML from file into XPathDocument
                using (FileStream documentStream = new FileStream(documentSource, FileMode.Open, FileAccess.Read, FileShare.Read))
                {
                    XmlTextReader documentReader = new XmlTextReader(documentStream);
                    xpathDocument = new XPathDocument(documentReader);
                }
            }

            //Checking if we have been given XslCompiledTransform directly, or do we have path for transform
            if ((transform == null) && (!String.IsNullOrEmpty(transformSource) && (transformSource.Trim().Length != 0)))
            {
                //Checking if path is absolute or relative
                if (!Path.IsPathRooted(transformSource))
                    //Mapping the relative path
                    transformSource = htmlHelper.ViewContext.HttpContext.Server.MapPath(transformSource);

                //Loading XSLT from file into XslCompiledTransform
                using (FileStream transformStream = new FileStream(transformSource, FileMode.Open, FileAccess.Read, FileShare.Read))
                {
                    XmlTextReader tranformReader = new XmlTextReader(transformStream);
                    transform = new XslCompiledTransform();
                    transform.Load(tranformReader);
                }
            }

            //Checking if we have XML in any form
            if (((xmlDocument != null) || (xpathDocument != null)) || (xpathNavigator != null))
            {
                //Checking if we have XSLT
                if (transform == null)
                    //If not, let's use transparent one
                    transform = _transparentTransform;

                //Perform transformation based on form in which we have our XML
                if (xmlDocument != null)
                    transform.Transform((IXPathNavigable)xmlDocument, transformArgumentList, writer);
                else if (xpathNavigator != null)
                    transform.Transform(xpathNavigator, transformArgumentList, writer);
                else
                    transform.Transform((IXPathNavigable)xpathDocument, transformArgumentList, writer);

                
            }
            
        }
        #endregion
    }
}